﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Enemy : MonoBehaviour {

	ChuckSubInstance myChuck;

	public int damageTaken = 0;
	public bool dying = false;
	public string my_pitch;
	public float my_freq_num;
	public string my_freq;
	public float wait_in_sec = 5.0f;
	public float projectile_speed = 2.0f;
	public GameObject ball_lightning = null;
	public GameObject shockwave = null;
	public GameObject bloodspray = null;

	private Dictionary<string, int> name2freq = new Dictionary<string, int>()
		{
			{"A", 440},
			{"B", 494},
			{"C", 262},
			{"D", 294},
			{"E", 330},
			{"F", 349},
			{"G", 392}
		};
	private Player_script player;
	private Animator anim;
	private Animator crystal_anim;
	private Spawner spawner;
	private float next_time = 0.0f;

	// Use this for initialization
	void Start () {
		myChuck = GetComponent<ChuckSubInstance>();
		anim = gameObject.GetComponent<Animator> ();
		crystal_anim = gameObject.transform.GetChild(0).GetComponent<Animator> ();
		my_pitch = getPitch ();
		my_freq_num = name2freq[my_pitch];
		int power = Random.Range (-1, 2);
		float octave = Mathf.Pow (2.0f,power);
		my_freq_num = my_freq_num * octave;
		my_freq = my_freq_num.ToString ();
		spawner = transform.root.gameObject.GetComponentInChildren<Spawner> ();
		player = transform.root.gameObject.GetComponentInChildren<Player_script> ();
	}
	
	// Update is called once per frame
	void Update () {
		if (Time.time > next_time && !dying) {
			Vector2 target_position = player.transform.position;
			fire (target_position);
			next_time = Time.time + wait_in_sec;
		}
	}

	void resonate (string pitch_class) {
		if (!dying && pitch_class == my_pitch) {
			Instantiate (shockwave, transform.position, Quaternion.identity);
			myChuck.RunCode( @"
				SqrOsc myOsc => ADSR e => dac;
				" + my_freq + @" => myOsc.freq;
				e.set( 10::ms, 40::ms, 0.0001, 0.01::ms );
				Noise n => ADSR a => HPF h => dac;
				2000.0 => h.freq;
				a.set(2::ms, 10::ms, 0.5, 0.9::ms);
				.5 => n.gain;
				fun void ksh () {
					25::ms => now;
					a.keyOn();
				    50::ms => now;
				    a.keyOff();
					1::ms => now;
				}
				100::ms => now;
				spork ~ ksh();
				for( 4 => int i; i > 0 ; i-- ) {
					0.1*i => myOsc.gain;
					e.keyOn();
					50::ms => now;
					e.keyOff();
					1::ms => now;
				}
				" );
			Vector2 sword_position = player.spell_pos;
			Vector2 my_position = new Vector2 (transform.position.x, transform.position.y);
			float the_distance = Vector2.Distance (sword_position, my_position);
			if (the_distance > 6f) {
				damageTaken += 1;
			} else if (the_distance > 3f) {
				damageTaken += 2;
			} else {
				damageTaken += 3;
			}
			crystal_anim.SetInteger ("damageTaken", damageTaken);
			if (damageTaken >= 3 && !dying) {
				Destroy (gameObject.transform.GetChild (1).gameObject);
				dying = true;
				anim.SetBool ("dying", dying);
				Instantiate (bloodspray, transform.position, Quaternion.identity);
				myChuck.RunCode( @"
					Noise n => ADSR e => HPF h => dac;
					2000.0 => h.freq;
					e.set(2::ms, 10::ms, 0.5, 0.9::ms);
					175::ms => now;
					for(3 => int i; i > 0; i--){
						0.2*i => n.gain;
						e.keyOn();
					    50::ms => now;
					    e.keyOff();
						1::ms => now;
					}
					" );
			}
		}
	}

	private void destructo () {
		spawner.concurrent_count--;
		player.score++;
		Destroy (gameObject);
	}

	private string getPitch(){
		List<string> pitch_names = new List<string> { "A", "B", "C", "D", "E", "F", "G" };
		return pitch_names[Random.Range(0, pitch_names.Count)];
	}

	void fire (Vector2 target_pos){
		Vector2 my_pos = new Vector2(transform.position.x,transform.position.y);
		Vector2 direction = target_pos - my_pos;
		direction.Normalize();
		Quaternion rotation = Quaternion.Euler( 0, 0, Mathf.Atan2 ( direction.y, direction.x ) * Mathf.Rad2Deg );
		GameObject projectile = (GameObject)Instantiate( ball_lightning, my_pos, rotation);
		projectile.GetComponent<Rigidbody2D>().velocity = direction * projectile_speed;
		myChuck.RunCode( @"
			SqrOsc myOsc => ADSR e => dac;
			" + my_freq + @" => myOsc.freq;
			e.set( 250::ms, 250::ms, 0.0001, 0.01::ms );
			0.1 => myOsc.gain;
			e.keyOn();
			500::ms => now;
			e.keyOff();
			1::ms => now;
			" );
	}
}
